(function () {
	var VISIBILITY_CHECKED_ROWS = "checked_rows";
	var VISIBILITY_NEVER = "never";

	/**
	 * To implement this interface you should implement 2 methods
	 * addictionalFilterAction
	 * getActionsToFilter
	 *  prepareActions
	 */
	window.Actionable = function Actionable() {
		function ensureIsArray(wantedAsArray) {
			var arrayResult = wantedAsArray;
			if (wantedAsArray === undefined) {
				arrayResult = [];
			} else if (!Util.isArray(wantedAsArray)) {
				arrayResult = [wantedAsArray];
			}
			return arrayResult;
		}
		/**
		 * The method will look for active widgets and return all actions
		 * in theirs that match with desired visibilities.
		 *
		 * @param array|string|undefined desiredVisibilities desired visibilities matched
		 *                                           by return actions for active
		 *                                           widgets; if undefined return all actions.
		 *
		 * @todo add list of visibilities
		 *
		 * @return array Returns an array of actions.
		 */
		var getActions = function (desiredVisibilities) {
			desiredVisibilities = ensureIsArray(desiredVisibilities);
			this.prepareActions();
			return this.filterActionsByVisibility(desiredVisibilities);
		};
		/**
		 * To interface Actionable
		 */
		this.getActionsToFilter = getActions;
		var filterByVisibilityRules = function (actions) {
			return actions.filter(function (action) {
				return action.aclIsVisible && action.isVisible;
			});
		};

		this.getActionsByVisibility = function (visibility) {
			var modes = this.getVisibilityModes(visibility);
			var actions = this.getActionsToFilter(modes);
			actions = filterByVisibilityRules(actions);
			return actions;
		};

		this.countActionsByVisibility = function (visibility) {
			return this.getActionsByVisibility(visibility).length;
		};

		this.getVisibilityModes = function (mode) {
			var visibilityModes = [mode, this.appendCurrentMode(mode)];
			if (mode === VISIBILITY_NEVER && this.hasCheckedRows()) {
				var neverCheckedRows = mode + "_" + VISIBILITY_CHECKED_ROWS;
				visibilityModes = visibilityModes.concat([neverCheckedRows, this.appendCurrentMode(neverCheckedRows)]);
			}
			return visibilityModes;
		};

		this.appendCurrentMode = function appendCurrentMode(mode) {
			return mode + "_" + this.getMode();
		};

		this.filterActionsByVisibility = function (desiredVisibilities) {
			return this.actions.filter((function (action) {
				var matched = true;
				var actionVisibilities = action.showAsAction.split("|");

				if (desiredVisibilities.length > 0) {
					matched = actionVisibilities.some(function (visibility) {
						return desiredVisibilities.indexOf(visibility) !== -1;
					});
				}
				return matched && this.addictionalFilterAction(action);
			}).bind(this));
		};
		this.getActionsTypeToShow = function () {
			if (this.hasCheckedRows()) {
				return VISIBILITY_CHECKED_ROWS;
			} else if (this.inInlineEditMode()) {
				return ViewMode.INLINE_EDIT;
			} else if (this.inEditMode()) {
				return ViewMode.EDIT;
			} else if (this.inViewMode()) {
				return ViewMode.VIEW;
			} else {
				return "always";
			}
		};
	};

})();